import numpy as np
import matplotlib.pyplot as plt
import scipy.integrate

#------------------------------------------------

def dsolve_Euler(f,x0,t):
   
    N=x0.shape[0]
    Nt=t.shape[0]
    
    x=np.zeros((Nt,N))
    x[0,:]=x0

    for j in range(Nt-1):
        h=t[j+1]-t[j]
        x[j+1,:]=x[j,:]+h*np.array(f(x[j,:],t[j]))
        
    return x

def dsolve_Heun(f,x0,t):
   
    N=x0.shape[0]
    Nt=t.shape[0]
    
    x=np.zeros((Nt,N))
    x[0,:]=x0

    for j in range(Nt-1):
        h=t[j+1]-t[j]
        k1=h*np.array(f(x[j,:],t[j]))
        k2=h*np.array(f(x[j,:]+k1,t[j]+h))
        x[j+1,:]=x[j,:]+(k1+k2)/2
        
    return x       
    
def dsolve_RungeKutta4(f,x0,t):
    
    N=x0.shape[0]
    Nt=t.shape[0]
    
    x=np.zeros((Nt,N))
    x[0,:]=x0

    for j in range(Nt-1):
        h=t[j+1]-t[j]
        k1=h*np.array(f(x[j,:],t[j]))
        k2=h*np.array(f(x[j,:]+k1/2,t[j]+h/2))
        k3=h*np.array(f(x[j,:]+k2/2,t[j]+h/2))
        k4=h*np.array(f(x[j,:]+k3,t[j]+h))
        x[j+1,:]=x[j,:]+(k1+2*k2+2*k3+k4)/6
        
    return x

#------------------------------------------------

def fun(x,t): 
    r=np.sqrt(x[0]**2+x[1]**2)
    return [x[2],x[3],-x[0]/r**3,-x[1]/r**3]

Nstep=1000
tmax=10.0

# initial conditions
r0=1.0
v0=0.5

x0=np.array([r0,0.0,0.0,v0])
t=np.linspace(0.0,tmax,Nstep+1)

# numerical solution
x=scipy.integrate.odeint(fun,x0,t)
#x=dsolve_Euler(fun,x0,t)
#x=dsolve_Heun(fun,x0,t)
#x=dsolve_RungeKutta4(fun,x0,t)

# exact trajectory
phi=np.linspace(0.0,2*np.pi,10000)
r=1/(1/(r0**4*v0**2)-(1/(r0**4*v0**2)-1/r0)*np.cos(phi))

# plot exact and numerical trajectory
plt.subplot(1,2,1)
plt.plot(r*np.cos(phi),r*np.sin(phi),'r-',x[:,0],x[:,1],'b-')
# plot coordinates and velocity components as functions of time
plt.subplot(1,2,2)
plt.plot(t,x[:,0],t,x[:,1],t,x[:,2],t,x[:,3])
plt.show()