import numpy as np
import matplotlib.pyplot as plt

#------------------------------------------------

def dsolve_Euler(f,x0,t):
   
    N=x0.shape[0]
    Nt=t.shape[0]
    
    x=np.zeros((Nt,N))
    x[0,:]=x0

    for j in range(Nt-1):
        h=t[j+1]-t[j]
        x[j+1,:]=x[j,:]+h*np.array(f(x[j,:],t[j]))
        
    return x

def dsolve_Heun(f,x0,t):
   
    N=x0.shape[0]
    Nt=t.shape[0]
    
    x=np.zeros((Nt,N))
    x[0,:]=x0

    for j in range(Nt-1):
        h=t[j+1]-t[j]
        k1=h*np.array(f(x[j,:],t[j]))
        k2=h*np.array(f(x[j,:]+k1,t[j]+h))
        x[j+1,:]=x[j,:]+(k1+k2)/2
        
    return x       
    
def dsolve_RungeKutta4(f,x0,t):
    
    N=x0.shape[0]
    Nt=t.shape[0]
    
    x=np.zeros((Nt,N))
    x[0,:]=x0

    for j in range(Nt-1):
        h=t[j+1]-t[j]
        k1=h*np.array(f(x[j,:],t[j]))
        k2=h*np.array(f(x[j,:]+k1/2,t[j]+h/2))
        k3=h*np.array(f(x[j,:]+k2/2,t[j]+h/2))
        k4=h*np.array(f(x[j,:]+k3,t[j]+h))
        x[j+1,:]=x[j,:]+(k1+2*k2+2*k3+k4)/6
        
    return x
    
#------------------------------------------------

def fun(x,t): return [x[0],-x[1]]
    
Nstep=20
tmax=1.0

x0=np.array([1.0,1.0])
t=np.linspace(0.0,tmax,Nstep+1)

x=dsolve_Euler(fun,x0,t)
#x=dsolve_Heun(fun,x0,t)
#x=dsolve_RungeKutta4(fun,x0,t)
print(x[Nstep,:])

# exact result for the last point
print(np.exp([1,-1]))

# plot exact and numerical ode solution
t2=np.linspace(0.0,tmax,1000)
plt.plot(t2,np.exp(t2),'r-',t2,np.exp(-t2),'b-',t,x[:,0],'r.',t,x[:,1],'b.')
plt.show()